<?php

namespace Database\Factories;

use App\Models\User;
use Illuminate\Database\Eloquent\Factories\Factory;
use Illuminate\Support\Str;

/**
 * @extends Factory<User>
 */
class UserFactory extends Factory
{
    /**
     * Define the model's default state.
     *
     * @return array<string, mixed>
     */
    public function definition(): array
    {
        return [
            'name' => fake()->name(),
            'email' => fake()->unique()->safeEmail(),
            'email_verified_at' => now(),
            'password' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi', // password
            'user_last_name_one' => fake()->lastName,
            'user_last_name_two' => fake()->lastName,
            'user_document_type' => fake()->numberBetween(1, 3),
            'user_document' => fake()->unique()->numerify('##########'),
            'user_birth' => fake()->date,
            'user_city' => fake()->city,
            'user_neighborhood' => fake()->city,
            'user_address' => fake()->address,
            'user_cellphone' => fake()->lastName,
            'user_photo' => fake()->lastName,
            'user_state' => fake()->lastName,
            'profile_code' => fake()->numberBetween(1, 3),
            'remember_token' => Str::random(10),
            'verified' => $verified = fake()->randomElement([User::USER_VERIFIED, User::USER_NOT_VERIFIED]),
            'verification_token' => $verified == User::USER_VERIFIED ? null : User::generateVerificationToken(),
        ];
    }

    /**
     * Indicate that the model's email address should be unverified.
     */
    public function unverified(): static
    {
        return $this->state(fn (array $attributes) => [
            'email_verified_at' => null,
        ]);
    }
}
